package nio;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousFileChannel;
import java.nio.channels.CompletionHandler;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public class AsyncReadExample {

	public static void main (String [] args) {
		new AsyncReadExample().readFile();
	}

	private void readFile() {

		String filePath = "test.txt";
		Path path = Paths.get(filePath);
		System.out.println();

		try(AsynchronousFileChannel channel =
			AsynchronousFileChannel.open(path, StandardOpenOption.READ)) {
			ByteBuffer buffer = ByteBuffer.allocate((int) channel.size()); // not for large files!

			// Method #1
			readFileFuture(channel, buffer);

			// Method #2
			readFileCompletion(channel, buffer);

		} catch (IOException | ExecutionException | InterruptedException e) {
			e.printStackTrace();
		}
	}

	private void readFileFuture(AsynchronousFileChannel channel, ByteBuffer buffer)
			throws InterruptedException, ExecutionException {

		// read to buffer, start from position 0
		Future<Integer> result = channel.read(buffer, 0);

		// wait until read
		System.out.println("Bytes read from file: " + result.get());
		printBuffer(buffer);
	}

	private void readFileCompletion(AsynchronousFileChannel channel, ByteBuffer buffer) {

		System.out.println();
		Thread currentThread = Thread.currentThread();
		channel.read(buffer, 0, buffer,
			new CompletionHandler<>() {
				@Override
				public void completed(Integer result, ByteBuffer attachment) {
					System.out.println("Read completed and " + result + " bytes are read. ");
					currentThread.interrupt();
				}

				@Override
				public void failed(Throwable e, ByteBuffer attachment) {
					System.out.println("Read failed with exception:");
					e.printStackTrace();
					System.exit(1);
				}
			});

		try {
			currentThread.join();
		} catch (InterruptedException e) {
			printBuffer(buffer);
		}
	}

	private void printBuffer(ByteBuffer buffer) {
		buffer.flip();
		System.out.println();
		System.out.println("Buffer contents: ");
		System.out.print(StandardCharsets.UTF_8.decode(buffer));
		System.out.println();
		System.out.println("--------------------------");
		buffer.clear();
	}
}
