package l11_io.nio;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousFileChannel;
import java.nio.channels.CompletionHandler;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public class AsyncWriteExample {

	public static void main(String[] args) {

		final String fileName = "test.txt";
		final int useFuture = args.length > 0 ? Integer.parseInt(args[0]) : 1;

		ByteBuffer byteBuffer = prepareBuffer();
		Path path = Paths.get(fileName);

		try (AsynchronousFileChannel channel = AsynchronousFileChannel.open(path,
				StandardOpenOption.CREATE, // create if file does not exist
				StandardOpenOption.WRITE, // writable
				StandardOpenOption.TRUNCATE_EXISTING)) {

			if (useFuture > 0) writeFileFuture(channel, byteBuffer);
			else writeFileCompletion(channel, byteBuffer);
		} catch (IOException | InterruptedException | ExecutionException e) {
			e.printStackTrace();
		}
		System.out.println();
		System.out.println("main(): after write");
		System.out.println(Thread.currentThread());
		System.out.println();
	}

	private static ByteBuffer prepareBuffer() {

		final String input = "Content to be written to the file\nAnd the second line";
		System.out.println("Input string:");
		System.out.println(input);
		System.out.println();

		return ByteBuffer.wrap(input.getBytes());
	}

	private static void writeFileFuture(AsynchronousFileChannel channel, ByteBuffer byteBuffer)
			throws ExecutionException, InterruptedException {

		Future<Integer> operation = channel.write(byteBuffer, 0);
		//byteBuffer.clear();
		int bytesWritten = operation.get();
		System.out.println("Future write done, " + bytesWritten + " bytes written");
	}

	private static void writeFileCompletion(AsynchronousFileChannel channel, ByteBuffer byteBuffer)
			throws InterruptedException {

		Thread currentThread = Thread.currentThread();
		CompletionHandler<Integer, ByteBuffer> handler = new CompletionHandler<>() {

			@Override
			public void completed(Integer result, ByteBuffer attachment) {
				System.out.println("Completion write done, " + result + " bytes written.");
//				nio.AsyncWriteExample.result = result; // to test completion
				System.out.println(Thread.currentThread());
				currentThread.interrupt();
			}

			@Override
			public void failed(Throwable e, ByteBuffer attachment) {
				System.out.println("Write failed with exception:");
				e.printStackTrace();
				System.exit(1);
			}
		};
		channel.write(byteBuffer, 0, byteBuffer, handler);
		System.out.println("Waiting for write to complete ...");
		System.out.println("You can switch to some other work here");
		System.out.println();
		try {
			currentThread.join();
		} catch (InterruptedException e) {
			System.out.println("Joined");
		}
	}
}